@@ -164,6 +164,7 @@
 #define PROFILE(x)
 #endif
 
+#define UPDATE_BRACKET_DELAY		50
 #define BRACKET_MATCHING_CHARS_LIMIT	10000
 #define CONTEXT_CLASSES_PREFIX		"gtksourceview:context-classes:"
 
@@ -217,6 +218,8 @@ struct _GtkSourceBufferPrivate
 
 	GtkTextTag *invalid_char_tag;
 
+	guint bracket_update_handler;
+
 	guint highlight_syntax : 1;
 	guint highlight_brackets : 1;
 	guint implicit_trailing_newline : 1;
@@ -424,7 +427,7 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 	 * updated (e.g. the no-spell-check context class).
 	 */
 	buffer_signals[HIGHLIGHT_UPDATED] =
-	    g_signal_new_class_handler ("highlight_updated",
+	    g_signal_new_class_handler ("highlight-updated",
 	                                G_OBJECT_CLASS_TYPE (object_class),
 	                                G_SIGNAL_RUN_LAST,
 	                                G_CALLBACK (gtk_source_buffer_real_highlight_updated),
@@ -439,11 +442,11 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 	 * @buffer: the buffer that received the signal
 	 * @mark: the #GtkSourceMark
 	 *
-	 * The ::source_mark_updated signal is emitted each time
+	 * The ::source-mark-updated signal is emitted each time
 	 * a mark is added to, moved or removed from the @buffer.
 	 */
 	buffer_signals[SOURCE_MARK_UPDATED] =
-	    g_signal_new ("source_mark_updated",
+	    g_signal_new ("source-mark-updated",
 			   G_OBJECT_CLASS_TYPE (object_class),
 			   G_SIGNAL_RUN_LAST,
 			   0,
@@ -592,6 +595,12 @@ gtk_source_buffer_dispose (GObject *object)
 	GtkSourceBuffer *buffer = GTK_SOURCE_BUFFER (object);
 	GList *l;
 
+	if (buffer->priv->bracket_update_handler != 0)
+	{
+		g_source_remove (buffer->priv->bracket_update_handler);
+		buffer->priv->bracket_update_handler = 0;
+	}
+
 	if (buffer->priv->undo_manager != NULL)
 	{
 		set_undo_manager (buffer, NULL);
@@ -895,6 +904,84 @@ bracket_pair (gunichar  base_char,
 	return pair;
 }
 
+/*
+ * This function works similar to gtk_text_buffer_remove_tag() except that
+ * instead of taking the optimization to make removing tags fast in terms
+ * of wall clock time, it tries to avoiding to much time of the screen
+ * by minimizing the damage regions. This results in fewer full-redraws
+ * when updating the text marks. To see the difference, compare this to
+ * gtk_text_buffer_remove_tag() and enable the "show pixel cache" feature
+ * the GTK+ inspector.
+ */
+static void
+remove_tag_with_minimal_damage (GtkTextBuffer     *buffer,
+                                GtkTextTag        *tag,
+                                const GtkTextIter *begin,
+                                const GtkTextIter *end)
+{
+	GtkTextIter tag_begin = *begin;
+	GtkTextIter tag_end;
+
+	if (!gtk_text_iter_starts_tag (&tag_begin, tag))
+	{
+		if (!gtk_text_iter_forward_to_tag_toggle (&tag_begin, tag))
+		{
+			return;
+		}
+	}
+
+	while (gtk_text_iter_starts_tag (&tag_begin, tag) &&
+	       gtk_text_iter_compare (&tag_begin, end) < 0)
+	{
+		gint count = 1;
+
+		tag_end = tag_begin;
+
+		/*
+		 * We might have found the start of another tag embedded
+		 * inside this tag. So keep scanning forward until we have
+		 * reached the right number of end tags.
+		 */
+
+		while (gtk_text_iter_forward_to_tag_toggle (&tag_end, tag))
+		{
+			if (gtk_text_iter_starts_tag (&tag_end, tag))
+			{
+				count++;
+			}
+			else if (gtk_text_iter_ends_tag (&tag_end, tag))
+			{
+				if (--count == 0)
+				{
+					break;
+				}
+			}
+		}
+
+		if (gtk_text_iter_ends_tag (&tag_end, tag))
+		{
+			gtk_text_buffer_remove_tag (buffer, tag, &tag_begin, &tag_end);
+
+			tag_begin = tag_end;
+
+			/*
+			 * Move to the next start tag. It's possible to have an overlapped
+			 * end tag, which would be non-ideal, but possible.
+			 */
+			if (!gtk_text_iter_starts_tag (&tag_begin, tag))
+			{
+				while (gtk_text_iter_forward_to_tag_toggle (&tag_begin, tag))
+				{
+					if (gtk_text_iter_starts_tag (&tag_begin, tag))
+					{
+						break;
+					}
+				}
+			}
+		}
+	}
+}
+
 static void
 update_bracket_highlighting (GtkSourceBuffer *source_buffer)
 {
@@ -913,10 +1000,10 @@ update_bracket_highlighting (GtkSourceBuffer *source_buffer)
 
 		gtk_text_buffer_get_bounds (buffer, &start, &end);
 
-		gtk_text_buffer_remove_tag (buffer,
-					    source_buffer->priv->bracket_match_tag,
-					    &start,
-					    &end);
+		remove_tag_with_minimal_damage (GTK_TEXT_BUFFER (source_buffer),
+		                                source_buffer->priv->bracket_match_tag,
+		                                &start,
+		                                &end);
 	}
 
 	if (!source_buffer->priv->highlight_brackets)
@@ -984,13 +1071,59 @@ update_bracket_highlighting (GtkSourceBuffer *source_buffer)
 	}
 }
 
+static gboolean
+do_bracket_update (gpointer user_data)
+{
+	GtkSourceBuffer *buffer = user_data;
+
+	buffer->priv->bracket_update_handler = 0;
+
+	update_bracket_highlighting (buffer);
+
+	return G_SOURCE_REMOVE;
+}
+
+static void
+queue_bracket_update (GtkSourceBuffer *buffer)
+{
+	if (buffer->priv->bracket_update_handler != 0)
+	{
+		g_source_remove (buffer->priv->bracket_update_handler);
+	}
+
+	/*
+	 * Queue an update to the bracket location instead of doing it
+	 * immediately. We are likely going to be servicing a draw deadline
+	 * immediately, so blocking to find the match and invalidating
+	 * visible regions causes animations to stutter. Instead, give
+	 * ourself just a little bit of a delay to catch up.
+	 *
+	 * The value for this delay was found experimentally, as 25msec
+	 * resulted in continuing to see frame stutter, but 50 was not
+	 * distinguishable from having matching-brackets disabled.
+	 * The animation in gtkscrolledwindow is 200, but that creates
+	 * an undesireable delay before the match is shown to the user.
+	 * 50msec errors on the side of "immediate", but without the
+	 * frame stutter.
+	 *
+	 * If we had access to a GdkFrameClock, we might consider using
+	 * ::update() or ::after-paint() to synchronize this.
+	 */
+	buffer->priv->bracket_update_handler =
+		gdk_threads_add_timeout_full (G_PRIORITY_LOW,
+					      UPDATE_BRACKET_DELAY,
+					      do_bracket_update,
+					      buffer,
+					      NULL);
+}
+
 /* Although this function is not really useful (update_bracket_highlighting()
  * could be called directly), the name cursor_moved() is more meaningful.
  */
 static void
 cursor_moved (GtkSourceBuffer *buffer)
 {
-	update_bracket_highlighting (buffer);
+	queue_bracket_update (buffer);
 }
 
 static void
@@ -998,7 +1131,7 @@ gtk_source_buffer_real_highlight_updated (GtkSourceBuffer *buffer,
 					  GtkTextIter     *start,
 					  GtkTextIter     *end)
 {
-	update_bracket_highlighting (buffer);
+	queue_bracket_update (buffer);
 }
 
 static void
