@@ -19,6 +19,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gtksourcestyle.h"
 #include "gtksourcestyle-private.h"
 #include "gtksourceview-i18n.h"
@@ -45,7 +49,7 @@ static void	gtk_source_style_get_property	(GObject      *object,
 
 struct _GtkSourceStyleClass
 {
-  GObjectClass parent_class;
+	GObjectClass parent_class;
 };
 
 G_DEFINE_TYPE (GtkSourceStyle, gtk_source_style, G_TYPE_OBJECT)
@@ -64,11 +68,14 @@ enum
 	PROP_ITALIC,
 	PROP_ITALIC_SET,
 	PROP_UNDERLINE,
+	PROP_PANGO_UNDERLINE,
 	PROP_UNDERLINE_SET,
 	PROP_STRIKETHROUGH,
 	PROP_STRIKETHROUGH_SET,
 	PROP_SCALE,
-	PROP_SCALE_SET
+	PROP_SCALE_SET,
+	PROP_UNDERLINE_COLOR,
+	PROP_UNDERLINE_COLOR_SET
 };
 
 static void
@@ -123,6 +130,11 @@ gtk_source_style_class_init (GtkSourceStyleClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	/**
+	 * GtkSourceStyle:underline
+	 *
+	 * Deprecated: 3.18: Use pango-underline.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_UNDERLINE,
 					 g_param_spec_boolean ("underline",
@@ -131,6 +143,15 @@ gtk_source_style_class_init (GtkSourceStyleClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property (object_class,
+					 PROP_PANGO_UNDERLINE,
+					 g_param_spec_enum ("pango-underline",
+							    "Pango Underline",
+							    "Pango Underline",
+							    PANGO_TYPE_UNDERLINE,
+							    PANGO_UNDERLINE_NONE,
+							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	g_object_class_install_property (object_class,
 					 PROP_STRIKETHROUGH,
 					 g_param_spec_boolean ("strikethrough",
@@ -147,6 +168,14 @@ gtk_source_style_class_init (GtkSourceStyleClass *klass)
 							      NULL,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property (object_class,
+					 PROP_UNDERLINE_COLOR,
+					 g_param_spec_string ("underline-color",
+							      "Underline Color",
+							      "Underline color",
+							      NULL,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	g_object_class_install_property (object_class,
 					 PROP_LINE_BACKGROUND_SET,
 					 g_param_spec_boolean ("line-background-set",
@@ -210,6 +239,14 @@ gtk_source_style_class_init (GtkSourceStyleClass *klass)
 							       "Whether scale attribute is set",
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					 PROP_UNDERLINE_COLOR_SET,
+					 g_param_spec_boolean ("underline-color-set",
+							       "Underline color set",
+							       "Whether underline color attribute is set",
+							       FALSE,
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
@@ -298,7 +335,12 @@ gtk_source_style_set_property (GObject      *object,
 			break;
 
 		case PROP_UNDERLINE:
-			style->underline = g_value_get_boolean (value) != 0;
+			style->underline = g_value_get_boolean (value) ? PANGO_UNDERLINE_SINGLE : PANGO_UNDERLINE_NONE;
+			SET_MASK (style, UNDERLINE);
+			break;
+
+		case PROP_PANGO_UNDERLINE:
+			style->underline = (PangoUnderline) g_value_get_enum (value);
 			SET_MASK (style, UNDERLINE);
 			break;
 
@@ -321,6 +363,20 @@ gtk_source_style_set_property (GObject      *object,
 			}
 			break;
 
+		case PROP_UNDERLINE_COLOR:
+			string = g_value_get_string (value);
+			if (string != NULL)
+			{
+				style->underline_color = g_intern_string (string);
+				SET_MASK (style, UNDERLINE_COLOR);
+			}
+			else
+			{
+				style->underline_color = NULL;
+				UNSET_MASK (style, UNDERLINE_COLOR);
+			}
+			break;
+
 		case PROP_FOREGROUND_SET:
 			MODIFY_MASK (style, value, FOREGROUND);
 			break;
@@ -348,10 +404,15 @@ gtk_source_style_set_property (GObject      *object,
 		case PROP_STRIKETHROUGH_SET:
 			MODIFY_MASK (style, value, STRIKETHROUGH);
 			break;
+
 		case PROP_SCALE_SET:
 			MODIFY_MASK (style, value, SCALE);
 			break;
 
+		case PROP_UNDERLINE_COLOR_SET:
+			MODIFY_MASK (style, value, UNDERLINE_COLOR);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -389,16 +450,25 @@ gtk_source_style_get_property (GObject      *object,
 			break;
 
 		case PROP_UNDERLINE:
-			g_value_set_boolean (value, style->underline);
+			g_value_set_boolean (value, style->underline != PANGO_UNDERLINE_NONE);
+			break;
+
+		case PROP_PANGO_UNDERLINE:
+			g_value_set_enum (value, style->underline);
 			break;
 
 		case PROP_STRIKETHROUGH:
 			g_value_set_boolean (value, style->strikethrough);
 			break;
+
 		case PROP_SCALE:
 			g_value_set_string (value, style->scale);
 			break;
 
+		case PROP_UNDERLINE_COLOR:
+			g_value_set_string (value, style->underline_color);
+			break;
+
 		case PROP_FOREGROUND_SET:
 			GET_MASK (style, value, FOREGROUND);
 			break;
@@ -426,10 +496,15 @@ gtk_source_style_get_property (GObject      *object,
 		case PROP_STRIKETHROUGH_SET:
 			GET_MASK (style, value, STRIKETHROUGH);
 			break;
+
 		case PROP_SCALE_SET:
 			GET_MASK (style, value, SCALE);
 			break;
 
+		case PROP_UNDERLINE_COLOR_SET:
+			GET_MASK (style, value, UNDERLINE_COLOR);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -463,6 +538,7 @@ gtk_source_style_copy (const GtkSourceStyle *style)
 	copy->italic = style->italic;
 	copy->bold = style->bold;
 	copy->underline = style->underline;
+	copy->underline_color = style->underline_color;
 	copy->strikethrough = style->strikethrough;
 	copy->mask = style->mask;
 	copy->scale = style->scale;
@@ -539,13 +615,25 @@ _gtk_source_style_apply (const GtkSourceStyle *style,
 
 		if (style->mask & GTK_SOURCE_STYLE_USE_UNDERLINE)
 		{
-			g_object_set (tag, "underline", style->underline ? PANGO_UNDERLINE_SINGLE : PANGO_UNDERLINE_NONE, NULL);
+			g_object_set (tag, "underline", style->underline, NULL);
 		}
 		else
 		{
 			g_object_set (tag, "underline-set", FALSE, NULL);
 		}
 
+		if (style->mask & GTK_SOURCE_STYLE_USE_UNDERLINE_COLOR)
+		{
+			GdkRGBA underline_rgba;
+
+			gdk_rgba_parse (&underline_rgba, style->underline_color);
+			g_object_set (tag, "underline-rgba", &underline_rgba, NULL);
+		}
+		else
+		{
+			g_object_set (tag, "underline-rgba-set", FALSE, NULL);
+		}
+
 		if (style->mask & GTK_SOURCE_STYLE_USE_STRIKETHROUGH)
 		{
 			g_object_set (tag, "strikethrough", style->strikethrough != 0, NULL);
@@ -610,6 +698,7 @@ _gtk_source_style_apply (const GtkSourceStyle *style,
 			      "style-set", FALSE,
 			      "weight-set", FALSE,
 			      "underline-set", FALSE,
+			      "underline-rgba-set", FALSE,
 			      "strikethrough-set", FALSE,
 			      "scale-set", FALSE,
 			      NULL);
